@@ -4,13 +4,16 @@ package Text::Xslate::PP;
 use 5.008;
 use strict;
 
-our $VERSION = '0.1023';
+our $VERSION = '0.1024';
 
 use Carp ();
 
 use Text::Xslate::PP::Const;
 use Text::Xslate::PP::State;
 use Text::Xslate::PP::EscapedString;
+use Text::Xslate::PP::Booster;
+
+use Text::Xslate::Util qw($DEBUG);
 
 my $TX_OPS = \%Text::Xslate::OPS;
 
@@ -85,10 +88,10 @@ sub _assemble {
     $self->{ template }->{ $name } = $tmpl;
     $self->{ tmpl_st }->{ $name }  = $st;
 
-    $tmpl->[ Text::Xslate::PP::Opcode::TXo_NAME ]      = $name;
-    $tmpl->[ Text::Xslate::PP::Opcode::TXo_MTIME ]     = $mtime;
-    $tmpl->[ Text::Xslate::PP::Opcode::TXo_CACHEPATH ] = $cachepath;
-    $tmpl->[ Text::Xslate::PP::Opcode::TXo_FULLPATH ]  = $fullpath;
+    $tmpl->[ Text::Xslate::PP::TXo_NAME ]      = $name;
+    $tmpl->[ Text::Xslate::PP::TXo_MTIME ]     = $mtime;
+    $tmpl->[ Text::Xslate::PP::TXo_CACHEPATH ] = $cachepath;
+    $tmpl->[ Text::Xslate::PP::TXo_FULLPATH ]  = $fullpath;
 
     $st->tmpl( $tmpl );
     $st->self( $self ); # weak_ref!
@@ -103,10 +106,10 @@ sub _assemble {
     $st->frame( [] );
     $st->current_frame( -1 );
 
-    my $mainframe = Text::Xslate::PP::Opcode::tx_push_frame( $st );
+    my $mainframe = tx_push_frame( $st );
 
-    $mainframe->[ Text::Xslate::PP::Opcode::TXframe_NAME ]    = 'main';
-    $mainframe->[ Text::Xslate::PP::Opcode::TXframe_RETADDR ] = $len;
+    $mainframe->[ Text::Xslate::PP::TXframe_NAME ]    = 'main';
+    $mainframe->[ Text::Xslate::PP::TXframe_RETADDR ] = $len;
 
     $st->lines( [] );
     $st->{ output } = '';
@@ -129,7 +132,6 @@ sub _assemble {
             Carp::croak( sprintf( "Oops: Unknown opcode '%s' on [%d]", $opname, $i ) );
         }
 
-        $code->[ $i ]->{ exec_code } = $Text::Xslate::PP::Opcode::Opcode_list->[ $opnum ];
         $code->[ $i ]->{ opname }    = $opname; # for test
 
         my $tx_oparg = $Text::Xslate::PP::tx_oparg->[ $opnum ];
@@ -183,7 +185,10 @@ sub _assemble {
         }
 
     }
-    $st->{code} = $code;
+
+    $st->{ perlcode } = Text::Xslate::PP::Booster->new()->opcode_to_perlcode( $proto );
+    $st->{ code     } = $code;
+    return;
 }
 
 
@@ -197,6 +202,23 @@ sub escaped_string {
 # INTERNAL
 #
 
+sub tx_push_frame {
+    my ( $st ) = @_;
+
+    if ( $st->current_frame > 100 ) {
+        Carp::croak("Macro call is too deep (> 100)");
+    }
+
+    $st->current_frame( $st->current_frame + 1 );
+
+    $st->frame->[ $st->current_frame ] ||= [];
+
+    $st->pad( $st->frame->[ $st->current_frame ] );
+
+    $st->frame->[ $st->current_frame ];
+}
+
+
 
 sub tx_load_template {
     my ( $self, $name ) = @_;
@@ -230,7 +252,7 @@ sub tx_load_template {
 
     my $tmpl = $ttobj->{ $name };
 
-    my $cache_mtime = $tmpl->[ Text::Xslate::PP::Opcode::TXo_MTIME ];
+    my $cache_mtime = $tmpl->[ Text::Xslate::PP::TXo_MTIME ];
 
     return $self->{ tmpl_st }->{ $name } unless $cache_mtime;
 
@@ -251,14 +273,14 @@ sub tx_all_deps_are_fresh {
     my ( $tmpl, $cache_mtime ) = @_;
     my $len = scalar @{$tmpl};
 
-    for ( my $i = Text::Xslate::PP::Opcode::TXo_FULLPATH; $i < $len; $i++ ) {
+    for ( my $i = Text::Xslate::PP::TXo_FULLPATH; $i < $len; $i++ ) {
         my $deppath = $tmpl->[ $i ];
 
         next unless defined $deppath;
 
         if ( ( stat( $deppath ) )[9] > $cache_mtime ) {
-            my $main_cache = $tmpl->[ Text::Xslate::PP::Opcode::TXo_CACHEPATH ];
-            if ( $i != Text::Xslate::PP::Opcode::TXo_FULLPATH and $main_cache ) {
+            my $main_cache = $tmpl->[ Text::Xslate::PP::TXo_CACHEPATH ];
+            if ( $i != Text::Xslate::PP::TXo_FULLPATH and $main_cache ) {
                 unlink $main_cache or warn $!;
             }
             return;
@@ -281,8 +303,8 @@ sub tx_execute { no warnings 'recursion';
 
     my $len = $st->code_len;
 
-    $st->{ output } = '';
-    $st->{ pc }     = 0;
+    $st->{output} = '';
+    $st->{pc}     = 0;
     $st->{vars}     = $vars;
 
     local $_depth      = $_depth + 1;
@@ -290,7 +312,7 @@ sub tx_execute { no warnings 'recursion';
 
     local $st->{local_stack};
 
-    $st->{code}->[ 0 ]->{ exec_code }->( $st );
+    $st->{perlcode}->( $st );
 
     $st->{targ} = undef;
     $st->{sa}   = undef;
@@ -309,13 +331,13 @@ sub _error_handler {
     Carp::croak( 'Not in $xslate->render()' ) unless $st;
 
     my $cframe = $st->frame->[ $st->current_frame ];
-    my $name   = $cframe->[ Text::Xslate::PP::Opcode::TXframe_NAME ];
+    my $name   = $cframe->[ Text::Xslate::PP::TXframe_NAME ];
 
     if( $die ) {
         $_depth = 0;
     }
 
-    my $file = $st->tmpl->[ Text::Xslate::PP::Opcode::TXo_NAME ];
+    my $file = $st->tmpl->[ Text::Xslate::PP::TXo_NAME ];
     my $line = $st->lines->[ $st->{ pc } ] || 0;
     my $mess = sprintf( "Xslate(%s:%d &%s[%d]): %s", $file, $line, $name, $st->{ pc }, $str );
 
@@ -358,7 +380,7 @@ Text::Xslate::PP - Yet another Text::Xslate runtime in pure Perl
 
 =head1 VERSION
 
-This document describes Text::Xslate::PP version 0.1023.
+This document describes Text::Xslate::PP version 0.1024.
 
 =head1 DESCRIPTION
 
@@ -377,6 +399,9 @@ If you want to use Text::Xslate::PP, however, you can use it.
 
 XS/PP mode might be switched with C<< $ENV{XSLATE} = 'pp' or 'xs' >>.
 
+From 0.1024 on, pure Perl version is not so slow.
+See L<Text::Xslate::PP::Booster> for details.
+
 =head1 SEE ALSO
 
 L<Text::Xslate>
