@@ -16,7 +16,7 @@ our @CARP_NOT = qw(Text::Xslate::Compiler Text::Xslate::Symbol);
 
 my $ID      = qr/(?: [A-Za-z_\$][A-Za-z0-9_]* )/xms;
 
-my $OPERATOR = sprintf '(?:%s)', join('|', map{ quotemeta } qw(
+my $OPERATOR_TOKEN = sprintf '(?:%s)', join('|', map{ quotemeta } qw(
     ...
     ..
     == != <=> <= >=
@@ -84,6 +84,13 @@ has token => (
     init_arg => undef,
 );
 
+has next_token => ( # to peep the next token
+    is  => 'rw',
+    isa => 'Maybe[ArrayRef]',
+
+    init_arg => undef,
+);
+
 has input => (
     is  => 'rw',
     isa => 'Str',
@@ -285,12 +292,12 @@ sub lex {
 
     local *_ = \$self->{input};
 
-    s{\G (\s) }{ $1 eq "\n" and $self->line_inc; ""}xmsge;
+    s{\G (\s) }{ $1 eq "\n" and $self->line_inc(); ""}xmsge;
 
     if(s/\A ($ID)//xmso){
         return [ name => $1 ];
     }
-    elsif(s/\A ($OPERATOR)//xmso){
+    elsif(s/\A ($OPERATOR_TOKEN)//xmso){
         return [ operator => $1 ];
     }
     elsif(s/\A $COMMENT //xmso) {
@@ -325,6 +332,7 @@ sub parse {
 
     $parser->input( $parser->preprocess($input) );
 
+    $parser->next_token( $parser->lex() );
     $parser->advance();
     my $ast = $parser->statements();
 
@@ -332,6 +340,7 @@ sub parse {
         $parser->_error("Syntax error", $parser->token);
     }
     $parser->near_token(undef);
+    $parser->next_token(undef);
 
     return $ast;
 }
@@ -507,17 +516,23 @@ sub advance {
 
     my $symtab = $parser->symbol_table;
 
-    $t = $parser->lex();
+    $t = $parser->next_token();
 
     if(not defined $t) {
         return $parser->token( $symtab->{"(end)"} );
     }
 
-    print STDOUT "[@{$t}]\n" if _DUMP_TOKEN;
+    $parser->next_token( $parser->lex() );
 
     my($arity, $value) = @{$t};
     my $proto;
 
+    if( $arity eq "name" && $parser->next_token->[1] eq "=>" ) {
+        $arity = "string";
+    }
+
+    print STDOUT "[$arity => $value]\n" if _DUMP_TOKEN;
+
     given($arity) {
         when("name") {
             $proto = $parser->find($value);
@@ -540,7 +555,7 @@ sub advance {
     }
 
     if(not defined $proto) {
-        $parser->_error("Unexpected token: $value ($arity)");
+        Carp::confess("Panic: Unexpected token: $value ($arity)");
     }
 
     return $parser->token( $proto->clone( id => $value, arity => $arity, line => $parser->line + 1 ) );
@@ -573,16 +588,7 @@ sub expression_list {
             push @args, $parser->expression(0);
             my $t = $parser->token;
 
-            if($t->id eq ",") {
-                # noop
-            }
-            elsif($t->id eq "=>") {
-                my $lhs = $args[-1];
-                if($lhs->arity ~~ [qw(name macro function)]) {
-                    $lhs->arity('literal');
-                }
-            }
-            else { # "," nor "=>"
+            if(!($t->id eq "," or $t->id eq "=>")) {
                 last;
             }
 
@@ -1178,16 +1184,13 @@ sub std_when {
 
 sub std_include {
     my($parser, $symbol) = @_;
-    my $args;
-    if($parser->token->id ne ";") {
-        $args = $parser->expression_list();
-    }
 
+    my $arg  = $parser->expression(0);
     my $vars = $parser->localize_vars();
 
     $parser->finish_statement();
     return $symbol->clone(
-        first  => $args,
+        first  => [$arg],
         second => $vars,
         arity  => 'command',
     );
@@ -1246,25 +1249,10 @@ sub _get_bare_name {
 sub localize_vars {
     my($parser) = @_;
     if($parser->token->id eq "{") {
-        my @vars;
         $parser->advance();
-
-        while(1) {
-            my $t = $parser->token;
-            last if $t->arity ne "name";
-            my $key = $t->id;
-
-            $parser->advance();
-
-            $parser->advance("=>");
-
-            push @vars, [ $key, $parser->expression(0) ];
-
-            last if $parser->token->id eq "}";
-            $parser->advance(",");
-        }
+        my $vars = $parser->expression_list();
         $parser->advance("}");
-        return \@vars;
+        return $vars;
     }
     return undef;
 }
@@ -1279,13 +1267,11 @@ sub std_cascade {
 
     my $components;
     if($parser->token->id eq "with") {
-        my @c;
         $parser->advance(); # "with"
 
-        push @c, $parser->_get_bare_name();
+        my @c = $parser->_get_bare_name();
         while($parser->token->id eq ",") {
             $parser->advance(); # ","
-
             push @c, $parser->_get_bare_name();
         }
         $components = \@c;
