@@ -1,11 +1,12 @@
 package Text::Xslate::Syntax::TTerse;
 use 5.010;
 use Any::Moose;
+use Text::Xslate::Util qw(p);
 
 extends qw(Text::Xslate::Parser);
 
 # [% ... %]
-sub _build_line_start { undef       }
+sub _build_line_start { qr/%%/xms   }
 sub _build_tag_start  { qr/\Q[%/xms }
 sub _build_tag_end    { qr/\Q%]/xms }
 
@@ -27,6 +28,9 @@ around split => sub {
 sub define_symbols {
     my($parser) = @_;
 
+    $parser->symbol(']');
+    $parser->symbol('}');
+
     $parser->define_basic_operators();
     $parser->infix('_', $parser->symbol('~')->lbp, \&led_concat);
 
@@ -51,8 +55,10 @@ sub define_symbols {
     $parser->symbol('FOR')     ->set_std(\&std_foreach);
     $parser->symbol('for')     ->set_std(\&std_foreach);
 
-    $parser->symbol('INCLUDE') ->set_std(\&std_command);
-    $parser->symbol('include') ->set_std(\&std_command);
+    $parser->symbol('INCLUDE') ->set_std(\&std_include);
+    $parser->symbol('include') ->set_std(\&std_include);
+    $parser->symbol('WITH');
+    $parser->symbol('with');
 
     return;
 }
@@ -153,13 +159,49 @@ sub std_foreach {
     return $proc;
 }
 
-sub std_command {
+sub std_include {
     my($parser, $symbol) = @_;
-    my $command = $parser->SUPER::std_command($symbol);
-    $command->id( lc( $command->id ) );
+
+    my $command = $parser->SUPER::std_include($symbol);
+    $command->id( lc $command->id );
     return $command;
 }
 
+sub localize_vars {
+    my($parser, $symbol) = @_;
+
+    if($parser->token->id eq "with") {
+        $parser->advance();
+        return  $parser->set_list();
+    }
+    return undef;
+}
+
+sub set_list {
+    my($parser) = @_;
+    my @args;
+    while(1) {
+        my $key = $parser->token;
+
+        if(!($key->arity eq "literal" || $key->arity eq "variable")) {
+            last;
+        }
+        $parser->advance();
+        $parser->advance("=");
+
+        my $value = $parser->expression(0);
+
+        $key->arity("literal");
+        push @args, $key => $value;
+
+        if($parser->token->id eq ",") { # , is optional
+            $parser->advance();
+        }
+    }
+
+    return \@args;
+}
+
 no Any::Moose;
 __PACKAGE__->meta->make_immutable();
 __END__
@@ -245,6 +287,14 @@ Almost the same as L<Text::Xslate::Syntax::Kolon>.
     [% INCLUDE "file.tt" %]
     [% INCLUDE $var %]
 
+C<< WITH variablies >> syntax is also supported:
+
+    [% INCLUDE "file.tt" WITH foo = 42, bar = 3.14 %]
+    [% INCLUDE "file.tt" WITH
+        foo = 42
+        bar = 3.14
+    %]
+
 =head2 Template cascading
 
 Not supported.
